#include <linux/config.h>
#include <linux/module.h>
#include <linux/fs.h>

#define WINEFS_MAGIC 0x57494E45

/* This is based on devpts and ramfs.
 * In particular, we follow ramfs in keeping everything in the dcache.
 */

static struct super_block *winefs_read_super(struct super_block *sb,
					     void *data, int silent)
{
    sb->s_blocksize = PAGE_CACHE_SIZE;
    sb->s_blocksize_bits = PAGE_CACHE_SHIFT;
    sb->s_magic = WINEFS_MAGIC;
    sb->s_op = &winefs_ops;

    inode = winefs_get_root_inode(sb);
    if (!inode) return NULL;

    root = d_alloc_root(inode);
    if (!root)
    {
	iput(inode);
	return NULL;
    }

    sb->s_root = root;
    return sb;
}

static int winefs_statfs(struct super_block *sb, struct statfs *stat)
{
    stat->f_type = sb->s_magic;
    stat->f_bsize = sb->s_blocksize;
    stat->f_namelen = WINEFS_MAX_NAME_LEN;
}

static struct super_operations winefs_ops =
{
    statfs: winefs_statfs,
    put_inode: force_delete, /* hmm */
};

static struct vfsmount *winefs_mnt;
static DECLARE_FSTYPE(winefs_type, "winefs", winefs_read_super,
		      FS_SINGLE|FS_LITTER);

static int __init register_and_mount_fs(struct filesystem_type* fs,
					struct vfsmount** mount_point)
{
    int err;

    err = register_filesystem(fs);
    if (!err)
    {
	*mount_point = kern_mount(fs);
	if (IS_ERR(*mount_point))
	{
	    err = PTR_ERR(*mount_point);
	    *mount_point = NULL;

	    unregister_filesystem(fs);
	}
    }

    return err;
}

static int __init winefs_init(void)
{
    return register_and_mount_fs(&winefs_type, &winefs_mnt);
}

static void __exit winefs_exit(void)
{
    kern_umount(winefs_mnt);
    unregister_filesystem(&winefs_type);
}

module_init(winefs_init);
module_exit(winefs_exit);
