#include <linux/config.h>
#include <linux/module.h>
#include <linux/fs.h>
#include <linux/string.h>
#include <linux/list.h>

struct file_operations winefs_root_operations =
{
    read: generic_read_dir,
    readdir: dcache_readdir,
};

struct inode_operations winefs_root_inode_operations =
{
    lookup: winefs_root_lookup,
};

struct winefs_object_type
{
    struct list_head list;

    const char *name;
    /* more fields to come */
};

static LIST_HEAD(winefs_objects);

static int winefs_create_root_entry(const char *name, struct inode *inode)
{
    struct dentry* dentry = d_alloc(sb->root, name);
    if (!dentry) return -ENOMEM;

    d_add(dentry, inode);
    d_get(dentry);
}

int winefs_register_object_type(const char *name)
{
    static ino_t next_inode = 16;

    struct winefs_object_type *type = kmalloc(sizeof(*type), GFP_KERNEL);
    if (!type) return -ENOMEM;

    list_add(&type->list, &winefs_objects);

    type->name = name;

    winefs_create_root_entry(name, winefs_make_subdir_inode(sb));
    winefs_create_root_entry(name + "_construct",
			     winefs_make_control_inode(sb));

    
}
